@@ -3,7 +3,7 @@ import itertools
 from collections.abc import Iterable
 import re
 import warnings
-from typing import Callable, Dict, Optional
+from typing import Callable, Optional, NamedTuple, Type
 
 import numpy as np
 import scipy
@@ -88,6 +88,13 @@ class Learner(ReprableWithPreprocessors):
     #: fitting the model
     preprocessors = ()
 
+    class FittedParameter(NamedTuple):
+        name: str
+        label: str
+        type: Type
+        min: Optional[int] = None
+        max: Optional[int] = None
+
     # Note: Do not use this class attribute.
     #       It remains here for compatibility reasons.
     learner_adequacy_err_msg = ''
@@ -179,6 +186,10 @@ class Learner(ReprableWithPreprocessors):
                 self.preprocessors is not type(self).preprocessors):
             yield from type(self).preprocessors
 
+    @property
+    def fitted_parameters(self) -> list:
+        return []
+
     # pylint: disable=no-self-use
     def incompatibility_reason(self, _: Domain) -> Optional[str]:
         """Return None if a learner can fit domain or string explaining why it can not."""
@@ -883,5 +894,5 @@ class XGBBase(SklLearner):
         self.params = kwargs
 
     @SklLearner.params.setter
-    def params(self, values: Dict):
+    def params(self, values: dict):
         self._params = values
