@@ -35,6 +35,15 @@ from Orange.widgets.widget import OWWidget, Msg
 
 log = logging.getLogger(__name__)
 
+# These two constants are used in settings (and in the proxy filter model).
+# The corresponding options in the combo box are translatable, therefore
+# the settings must be stored in language-independent form.
+GENERAL_DOMAIN = None
+ALL_DOMAINS = ""  # The setting is Optional[str], so don't use other types here
+
+# The number of characters at which filter overrides the domain and language
+FILTER_OVERRIDE_LENGTH = 4
+
 
 def ensure_local(index_url, file_path, local_cache_path,
                  force=False, progress_advance=None):
@@ -102,6 +111,8 @@ class UniformHeightIndicatorDelegate(
 
 
 class Namespace(SimpleNamespace):
+    PUBLISHED, UNLISTED = range(2)
+
     def __init__(self, **kwargs):
         self.file_path = None
         self.prefix = None
@@ -122,8 +133,10 @@ class Namespace(SimpleNamespace):
         self.seealso = []
         self.tags = []
         self.language = "English"
+        self.domain = None
+        self.publication_status = self.PUBLISHED
 
-        super(Namespace, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
         # if title missing, use filename
         if not self.title and self.filename:
@@ -144,6 +157,12 @@ class SortFilterProxyWithLanguage(QSortFilterProxyModel):
     def __init__(self):
         super().__init__()
         self.__language = None
+        self.__domain = None
+        self.__filter = None
+
+    def setFilterFixedString(self, pattern):
+        self.__filter = pattern and pattern.casefold()
+        super().setFilterFixedString(pattern)
 
     def setLanguage(self, language):
         self.__language = language
@@ -152,13 +171,27 @@ class SortFilterProxyWithLanguage(QSortFilterProxyModel):
     def language(self):
         return self.__language
 
+    def setDomain(self, domain):
+        self.__domain = domain
+        self.invalidateFilter()
+
+    def domain(self):
+        return self.__domain
+
     def filterAcceptsRow(self, row, parent):
         source = self.sourceModel()
-        return super().filterAcceptsRow(row, parent) and (
-            self.__language is None
-            or source.index(row, 0).data(Qt.UserRole).language == self.__language
+        data = source.index(row, 0).data(Qt.UserRole)
+        in_filter = (
+            self.__filter is not None
+            and len(self.__filter) >= FILTER_OVERRIDE_LENGTH
+            and self.__filter in data.title.casefold()
         )
-
+        published_ok = data.publication_status == Namespace.PUBLISHED
+        domain_ok = self.__domain in (ALL_DOMAINS, data.domain)
+        language_ok = self.__language in (None, data.language)
+        return (super().filterAcceptsRow(row, parent)
+            and (published_ok and domain_ok and language_ok
+                 or in_filter))
 
 class OWDataSets(OWWidget):
     name = "Datasets"
@@ -178,6 +211,10 @@ class OWDataSets(OWWidget):
     DEFAULT_LANG = "English"
     ALL_LANGUAGES = "All Languages"
 
+    # These two combo options are translatable; others (domain names) are not
+    GENERAL_DOMAIN_LABEL = "(General)"
+    ALL_DOMAINS_LABEL = "(Show all)"
+
     # override HEADER_SCHEMA to define new columns
     # if schema is changed override methods: self.assign_delegates and
     # self.create_model
@@ -204,8 +241,11 @@ class OWDataSets(OWWidget):
         data = Output("Data", Orange.data.Table)
 
     #: Selected dataset id
-    selected_id = Setting(None)   # type: Optional[str]
+    selected_id: Optional[str] = Setting(None)
     language = Setting(DEFAULT_LANG)
+    domain = Setting(GENERAL_DOMAIN)
+    filter_hint: Optional[str] = Setting(None)
+    settings_version = 2
 
     #: main area splitter state
     splitter_state = Setting(b'')  # type: bytes
@@ -234,17 +274,41 @@ class OWDataSets(OWWidget):
         self.filterLineEdit = QLineEdit(
             textChanged=self.filter, placeholderText="Search for data set ..."
         )
+        self.filterLineEdit.setToolTip(
+            "Typing four letters or more overrides domain and language filters")
         layout.addWidget(self.filterLineEdit)
+
+        self.combo_elements = []
+
         layout.addSpacing(20)
-        layout.addWidget(QLabel("Show data sets in "))
+        label = QLabel("Show data sets in ")
+        layout.addWidget(label)
+        self.combo_elements.append(label)
+
         lang_combo = self.language_combo = QComboBox()
         languages = [self.DEFAULT_LANG, self.ALL_LANGUAGES]
         if self.language is not None and self.language not in languages:
             languages.insert(1, self.language)
         lang_combo.addItems(languages)
-        lang_combo.setCurrentText(self.language)
+        if self.language is None:
+            lang_combo.setCurrentIndex(lang_combo.count() - 1)
+        else:
+            lang_combo.setCurrentText(self.language)
         lang_combo.activated.connect(self._on_language_changed)
         layout.addWidget(lang_combo)
+        self.combo_elements.append(lang_combo)
+
+        domain_combo = self.domain_combo = QComboBox()
+        domain_combo.addItem(self.GENERAL_DOMAIN_LABEL)
+        domain_combo.activated.connect(self._on_domain_changed)
+        if self.core_widget:
+            layout.addSpacing(20)
+            label = QLabel("Domain:")
+            layout.addWidget(label)
+            self.combo_elements.append(label)
+            layout.addWidget(domain_combo)
+        self.combo_elements.append(domain_combo)
+
         self.mainArea.layout().addLayout(layout)
 
         self.splitter = QSplitter(orientation=Qt.Vertical)
@@ -290,6 +354,9 @@ class OWDataSets(OWWidget):
         proxy.setFilterKeyColumn(-1)
         proxy.setFilterCaseSensitivity(Qt.CaseInsensitive)
         self.view.setModel(proxy)
+        if not self.core_widget:
+            self.domain = ALL_DOMAINS
+            self.view.model().setDomain(self.domain)
 
         if self.splitter_state:
             self.splitter.restoreState(self.splitter_state)
@@ -306,6 +373,18 @@ class OWDataSets(OWWidget):
 
         self._on_language_changed()
 
+    # Single cell add-on has a data set widget that derives from this one
+    # although this class isn't defined as open. Adding the domain broke
+    # single-cell. A proper solution would be to split this widget into an
+    # (open) base class and a closed widget that adds the domain functionality.
+    # Yet, simply excluding three chunks of code makes this code simpler
+    # - which is better, if we assume that extending this widget is an anomaly.
+    @property
+    def core_widget(self):
+        # Compare by names; unit tests wrap widget classes in to detect
+        # missing onDeleteWidget calls
+        return type(self).__name__ == OWDataSets.__name__
+
     def assign_delegates(self):
         # NOTE: All columns must have size hinting delegates.
         # QTreeView queries only the columns displayed in the viewport so
@@ -353,6 +432,7 @@ class OWDataSets(OWWidget):
 
     def create_model(self):
         self.update_language_combo()
+        self.update_domain_combo()
         return self.update_model()
 
     def update_language_combo(self):
@@ -367,13 +447,22 @@ class OWDataSets(OWWidget):
         if self.DEFAULT_LANG not in languages:
             combo.addItem(self.DEFAULT_LANG)
         combo.addItems(languages + [self.ALL_LANGUAGES])
-        if current_language in languages:
+        if current_language in languages or current_language == self.ALL_LANGUAGES:
             combo.setCurrentText(current_language)
         elif self.DEFAULT_LANG in languages:
             combo.setCurrentText(self.DEFAULT_LANG)
         else:
             combo.setCurrentText(self.ALL_LANGUAGES)
 
+    def update_domain_combo(self):
+        combo = self.domain_combo
+        allkeys = set(self.allinfo_local) | set(self.allinfo_remote)
+        domains = {self._parse_info(key).domain for key in allkeys}
+        domains -= {None, "sc"}
+        if domains:
+            combo.addItems(sorted(domains))
+            combo.addItem(self.ALL_DOMAINS_LABEL)
+
     def update_model(self):
         allkeys = set(self.allinfo_local) | set(self.allinfo_remote)
         allkeys = sorted(allkeys)
@@ -410,11 +499,26 @@ class OWDataSets(OWWidget):
             model.appendRow(row)
 
             # for settings do not use os.path.join (Windows separator is different)
-            if "/".join(file_path) == self.selected_id:
+            if file_path[-1] == self.selected_id:
                 current_index = i
+                if self.core_widget:
+                    self.domain = datainfo.domain
+                    if self.domain == "sc":  # domain from the list of ignored domain
+                        self.domain = ALL_DOMAINS
+                    self.__update_domain_combo()
+                    self._on_domain_changed()
 
         return model, current_index
 
+    def __update_domain_combo(self):
+        combo = self.domain_combo
+        if self.domain == GENERAL_DOMAIN:
+            combo.setCurrentIndex(0)
+        elif self.domain == ALL_DOMAINS:
+            combo.setCurrentIndex(combo.count() - 1)
+        else:
+            combo.setCurrentText(self.domain)
+
     def _on_language_changed(self):
         combo = self.language_combo
         if combo.currentIndex() == combo.count() - 1:
@@ -423,6 +527,16 @@ class OWDataSets(OWWidget):
             self.language = combo.currentText()
         self.view.model().setLanguage(self.language)
 
+    def _on_domain_changed(self):
+        combo = self.domain_combo
+        if combo.currentIndex() == 0:
+            self.domain = GENERAL_DOMAIN
+        elif combo.currentIndex() == combo.count() - 1:
+            self.domain = ALL_DOMAINS
+        else:
+            self.domain = combo.currentText()
+        self.view.model().setDomain(self.domain)
+
     @Slot(object)
     def __set_index(self, f):
         # type: (Future) -> None
@@ -448,6 +562,16 @@ class OWDataSets(OWWidget):
 
     def set_model(self, model, current_index):
         self.view.model().setSourceModel(model)
+        if current_index != -1:
+            for hint in (
+                    self.filter_hint,
+                    model.index(current_index, 0).data(Qt.UserRole).title):
+                if self.view.model().filterAcceptsRow(current_index,
+                                                      QModelIndex()):
+                    break
+                self.filterLineEdit.setText(hint)
+                self.filter()
+
         self.view.selectionModel().selectionChanged.connect(
             self.__on_selection
         )
@@ -512,6 +636,18 @@ class OWDataSets(OWWidget):
 
     def filter(self):
         filter_string = self.filterLineEdit.text().strip()
+        enable_combos = len(filter_string) < FILTER_OVERRIDE_LENGTH
+        if enable_combos is not self.domain_combo.isEnabled():
+            for element in self.combo_elements:
+                element.setEnabled(enable_combos)
+            if enable_combos:
+                self.__update_domain_combo()
+                self.language_combo.setCurrentText(self.language)
+            else:
+                self.domain_combo.setCurrentText(self.ALL_DOMAINS_LABEL)
+                self.language_combo.setCurrentText(self.ALL_LANGUAGES)
+
+        self.filter_hint = filter_string
         proxyModel = self.view.model()
         if proxyModel:
             proxyModel.setFilterFixedString(filter_string)
@@ -526,11 +662,8 @@ class OWDataSets(OWWidget):
             di = current.data(Qt.UserRole)
             text = description_html(di)
             self.descriptionlabel.setText(text)
-            # for settings do not use os.path.join (Windows separator is different)
-            self.selected_id = "/".join(di.file_path)
         else:
             self.descriptionlabel.setText("")
-            self.selected_id = None
 
     def commit(self):
         """
@@ -543,6 +676,7 @@ class OWDataSets(OWWidget):
         di = self.selected_dataset()
         if di is not None:
             self.Error.clear()
+            self.selected_id = di.file_path[-1]
 
             if self.__awaiting_state is not None:
                 # disconnect from the __commit_complete
@@ -558,8 +692,7 @@ class OWDataSets(OWWidget):
                 self.__awaiting_state = None
 
             if not di.islocal:
-                pr = progress()
-                callback = lambda pr=pr: pr.advance.emit()
+                pr = Progress()
                 pr.advance.connect(self.__progress_advance, Qt.QueuedConnection)
 
                 self.progressBarInit()
@@ -569,7 +702,7 @@ class OWDataSets(OWWidget):
                 f = self._executor.submit(
                     ensure_local, self.INDEX_URL, di.file_path,
                     self.local_cache_path, force=di.outdated,
-                    progress_advance=callback)
+                    progress_advance=pr.advance.emit)
                 w = FutureWatcher(f, parent=self)
                 w.done.connect(self.__commit_complete)
                 self.__awaiting_state = _FetchState(f, w, pr)
@@ -578,6 +711,7 @@ class OWDataSets(OWWidget):
                 self.setBlocking(False)
                 self.commit_cached(di.file_path)
         else:
+            self.selected_id = None
             self.load_and_output(None)
 
     @Slot(object)
@@ -620,8 +754,7 @@ class OWDataSets(OWWidget):
             self.__awaiting_state.pb.advance.disconnect(self.__progress_advance)
             self.__awaiting_state = None
 
-    @staticmethod
-    def sizeHint():
+    def sizeHint(self):
         return QSize(1100, 500)
 
     def closeEvent(self, event):
@@ -644,10 +777,13 @@ class OWDataSets(OWWidget):
         return Orange.data.Table(path)
 
     @classmethod
-    def migrate_settings(cls, settings, _):
-        # until including 3.36.0 selected dataset was saved with \ on Windows
-        if "selected_id" in settings and isinstance(settings["selected_id"], str):
-            settings["selected_id"] = settings["selected_id"].replace("\\", "/")
+    def migrate_settings(cls, settings, version: Optional[int] = None):
+        selected_id = settings.get("selected_id")
+        if isinstance(selected_id, str):
+            # until including 3.36.0 selected dataset was saved with \ on Windows
+            selected_id = selected_id.replace("\\", "/")
+            if version is None or version < 2:
+                settings["selected_id"] = selected_id.split("/")[-1]
 
 
 class FutureWatcher(QObject):
@@ -666,7 +802,7 @@ class FutureWatcher(QObject):
         self.done.emit(self.__future)
 
 
-class progress(QObject):
+class Progress(QObject):
     advance = Signal()
 
 
@@ -692,7 +828,7 @@ def make_html_list(items):
     style = '"margin: 5px; text-indent: -40px; margin-left: 40px;"'
 
     def format_item(i):
-        return '<p style={}><small>{}</small></p>'.format(style, i)
+        return f'<p style={style}><small>{i}</small></p>'
 
     return '\n'.join([format_item(i) for i in items])
 
@@ -703,11 +839,11 @@ def description_html(datainfo):
     Summarize a data info as a html fragment.
     """
     html = []
-    year = " ({})".format(str(datainfo.year)) if datainfo.year else ""
-    source = ", from {}".format(datainfo.source) if datainfo.source else ""
+    year = f" ({datainfo.year})" if datainfo.year else ""
+    source = f", from {datainfo.source}" if datainfo.source else ""
 
-    html.append("<b>{}</b>{}{}".format(escape(datainfo.title), year, source))
-    html.append("<p>{}</p>".format(datainfo.description))
+    html.append(f"<b>{escape(datainfo.title)}</b>{year}{source}")
+    html.append(f"<p>{datainfo.description}</p>")
     seealso = make_html_list(datainfo.seealso)
     if seealso:
         html.append("<small><b>See Also</b>\n" + seealso + "</small>")
